//+------------------------------------------------------------------+
//|                                                 Projection bands |
//|                                                                  |
//|                                                                  |
//| original indicator described in TASC 13:7 article                |
//| "Signaling Change With Projection Bands" by Mel Widner Ph.D.     }
//|                                                                  |
//+------------------------------------------------------------------+

#property copyright "mladen"
#property link      "mladenfx@gmail.com"

#property indicator_separate_window
#property  indicator_buffers 2
#property  indicator_color1  DeepSkyBlue
#property  indicator_color2  PaleVioletRed
#property  indicator_width2  2
//#property  indicator_style1  STYLE_DOT

//
//
//
//
//

extern int    ProjectionPeriod  = 20;
extern double SignalPeriod      =  3;

//
//
//
//
//

double   bufferPoc[];
double   bufferSig[];

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int init()
{
   SetIndexBuffer(0,bufferPoc);
   SetIndexBuffer(1,bufferSig);
         IndicatorShortName("Projection oscilator ("+ProjectionPeriod+","+DoubleToStr(SignalPeriod,2)+")");
   return(0);
}
int deinit() { return(0); }


//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int start()
{
   int counted_bars=IndicatorCounted();
   int limit,i;
   
   if(counted_bars<0) return(-1);
   if(counted_bars>0) counted_bars--;
           limit=MathMin(Bars-counted_bars,Bars-1);

   //
   //
   //
   //
   //

   double alpha = 2.0/(1.0+SignalPeriod);
   for(i=limit; i>=0; i--)
   {
      double slopel = lrsSlope(ProjectionPeriod,PRICE_LOW ,i);
      double slopeh = lrsSlope(ProjectionPeriod,PRICE_HIGH,i);
         double max = iMA(NULL,0,1,0,MODE_SMA,PRICE_HIGH,i);
         double min = iMA(NULL,0,1,0,MODE_SMA,PRICE_LOW ,i);

         for (int k=1; k<ProjectionPeriod; k++)
         {
            min = MathMin(min,iMA(NULL,0,1,0,MODE_SMA,PRICE_LOW ,i+k)+k*slopel);
            max = MathMax(max,iMA(NULL,0,1,0,MODE_SMA,PRICE_HIGH,i+k)+k*slopeh);
         }
         if (max!=min)
               bufferPoc[i] = 100.0*(iMA(NULL,0,1,0,MODE_SMA,PRICE_CLOSE,i)-min)/(max-min);
         else  bufferPoc[i] = 0;
               bufferSig[i] = bufferSig[i+1]+alpha*(bufferPoc[i]-bufferSig[i+1]);
   }
   
   //
   //
   //
   //
   //
   
   return(0);
}


  
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//

double lrsSlope(int period,int appliedPrice,int shift)
{
   double sumx  = period*(period+1.0)/2.0;
   double sumx2 = period*(period+1.0)*(2*period+1.0)/6.0;
   double sumxy = 0.00;
   double sumy  = 0.00;

   int n = period;
   int i;
      for (i=0; i<period; i++,n--)
      {
         double price  = iMA(NULL,0,1,0,MODE_SMA,appliedPrice,shift+i);
                sumxy += n*price;
                sumy  +=   price;
      }        

   //
   //
   //
   //
   //
   
   double slope = 0.00;
   double top   = period*sumxy - sumx*sumy;
   double bot   = period*sumx2 - sumx*sumx;
      if (bot != 0) slope = top/bot;
   return(slope);
}